package java.time.chrono;

import java.time.format.DateTimeFormatterBuilder;
import java.util.Locale;
import java.time.format.TextStyle;
import java.time.temporal.Temporal;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalQueries;
import java.time.temporal.TemporalQuery;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.time.temporal.ValueRange;
import java.time.temporal.ChronoField;
import java.time.temporal.TemporalField;
import java.time.temporal.TemporalAdjuster;
import java.time.temporal.TemporalAccessor;

public interface Era extends TemporalAccessor, TemporalAdjuster
{
    public abstract int getValue();
    
    default public boolean isSupported(TemporalField temporalField) {
        if ((temporalField instanceof ChronoField)) {
            return temporalField == ChronoField.ERA;
        }
        return temporalField != null && temporalField.isSupportedBy(this);
    }
    
    default public ValueRange range(TemporalField field) {
        return super.range(field);
    }
    
    default public int get(TemporalField field) {
        if (field == ChronoField.ERA) {
            return this.getValue();
        }
        return super.get(field);
    }
    
    default public long getLong(TemporalField obj) {
        if (obj == ChronoField.ERA) {
            return (long)this.getValue();
        }
        if ((obj instanceof ChronoField)) {
            throw new UnsupportedTemporalTypeException("Unsupported field: " + obj);
        }
        return obj.getFrom(this);
    }
    
    default public <R> R query(TemporalQuery<R> query) {
        if (query == TemporalQueries.precision()) {
            return ChronoUnit.ERAS;
        }
        return super.query(query);
    }
    
    default public Temporal adjustInto(Temporal temporal) {
        return temporal.with(ChronoField.ERA, (long)this.getValue());
    }
    
    default public String getDisplayName(TextStyle textStyle, Locale locale) {
        return new DateTimeFormatterBuilder().appendText(ChronoField.ERA, textStyle).toFormatter(locale).format(this);
    }
}
